﻿// =============================================================================
// KitaLab USB to I2C モジュール サンプルプログラム１
// -----------------------------------------------------------------------------
// [動作OS]
// ・Windows XP (SP3)
// ・Windows 7 (32bit/64bit)
// [使用ツール/媒体]
// ・Microsoft Visual C# (開発はC#2008 Professional)
// ・KitaLab製 USB to I2Cモジュール
// ・KitaLab製 40連LEDバーモジュール×１個 (I2Cスレーブアドレス=0xD0)
// -----------------------------------------------------------------------------
// [使用方法]
// １．USB to I2C モジュールに40連LEDモジュールを接続する。
// ２．USB to I2C モジュールをPCに接続する。
//     (HIDデバイスとして認識されるのでドライバ不要)
// ３．test01.exeを起動する。
// ４．「TEST 1」ボタンを押下して40連LEDモジュールが点灯すれば動作OK。
// -----------------------------------------------------------------------------
// Copyright(C) 2013 KitaLab
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace test01
{
	public partial class Form1 : Form
	{
		// USBtoI2C DLL関数準備
		[DllImport("Usb2I2c.dll")]
		static extern uint i2c_error();
		[DllImport("Usb2I2c.dll")]
		static extern uint i2c_ledind(bool sw);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_start();
		[DllImport("Usb2I2c.dll")]
		static extern byte i2c_read(bool ack);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_write(byte data);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_stop();
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_buff_write(byte slave, byte length, byte[] data);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_text_write(byte slave, string text);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_text_prog(string text, byte[] result, uint bufsize, ref uint resultsize);

		byte m_nSlave = 0xd0;

		// コンストラクタ
		public Form1()
		{
			InitializeComponent();
		}

		// LED ON
		// 基板の通信ステータスLEDを点灯させるよう設定
		private void button3_Click(object sender, EventArgs e)
		{
			i2c_ledind(true);
		}

		// LED OFF
		// 基板の通信ステータスLEDを点灯させないよう設定
		private void button4_Click(object sender, EventArgs e)
		{
			i2c_ledind(false);
		}

		// テキスト一括送信テスト
		private void button1_Click(object sender, EventArgs e)
		{
			Cursor preCursor = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			start_setup();

			for (int j = 0; j < 3; j++)
			{
				for (int i = 1; i <= 40; i++)
				{
					i2c_text_write(m_nSlave, "00" + (i).ToString("x2"));
				}
				for (int i = 39; i >= 0; i--)
				{
					i2c_text_write(m_nSlave, "00" + (i).ToString("x2"));
				}
			}

			end_setup();
			Cursor.Current = preCursor;
		}

		// バイナリ一括送信テスト
		private void button5_Click(object sender, EventArgs e)
		{
			Cursor preCursor = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			start_setup();

			byte[] send = new byte[2] { 0x00, 0x00 };
			for (int j = 0; j < 3; j++)
			{
				for (int i = 1; i <= 40; i++)
				{
					send[1] = (byte)i;
					i2c_buff_write(m_nSlave, 2, send);
				}
				for (int i = 39; i >= 0; i--)
				{
					send[1] = (byte)i;
					i2c_buff_write(m_nSlave, 2, send);
				}
			}

			end_setup();
			Cursor.Current = preCursor;
		}

		// I2Cの基本命令のみで通信するテスト
		private void button2_Click(object sender, EventArgs e)
		{
			Cursor preCursor = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			start_setup();

			for (int j = 0; j < 3; j++)
			{
				for (int i = 1; i <= 40; i++)
				{
					i2c_start();
					i2c_write(m_nSlave);
					i2c_write(0x00);
					i2c_write((byte)i);
					i2c_stop();
				}
				for (int i = 39; i >= 0; i--)
				{
					i2c_start();
					i2c_write(m_nSlave);
					i2c_write(0x00);
					i2c_write((byte)i);
					i2c_stop();
				}
			}

			end_setup();
			Cursor.Current = preCursor;
		}

		// 受信テスト
		private void button6_Click(object sender, EventArgs e)
		{
			Cursor preCursor = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			start_setup();

			// 40連LEDアレイモジュールのポイントアドレス0x02から6bytes分の設定値を取得するサンプル。
			// 直上のstart_setup();により0x80 0x00 0x00 0x28 0x28 0x00の6bytesが取得できるはずである。
			// -----------
			// 複雑に見えるが、スレーブアドレスをリテラルで表すと "Wd0 W02 s Wd1 RRRRRF" となる。
			// Wはwrite、sはstartコンディション、RとFは受信指示で、RがAck有り、FがAck無し指示。
			// -----------
			// Wd0でスレーブデバイスを選択し、W02でデバイス固有のポイントアドレス0x02を指定。
			// sでリスタートし、Wd1で0xd0のRead指示(0bit目を立てる)。RRRRRFで6bytes分取得。
			// 先頭にs(start condition)、末尾にe(stop condition)が本来必要だが、省略すると
			// 自動的に付加されるので省略しても良い。
			// -----------
			// 戻り値のresBufには、RとFの数だけ返る。実際に戻されたバイト数はpResSizeに入る。

			byte[] resBuf = new byte[64];
			uint nBufSize = (uint)resBuf.Length;
			uint pResSize = 0;
			System.Threading.Thread.Sleep(1);
			bool bRet = i2c_text_prog("W" + (m_nSlave).ToString("x2") + "W02 s W" + (m_nSlave + 1).ToString("x2") + "RRRRRF", resBuf, nBufSize, ref pResSize);

			end_setup();
			Cursor.Current = preCursor;

			// 結果表示
			if (bRet)
			{
				string txt = "";
				txt += "計 " + (pResSize).ToString() + "bytes\n\n";
				for (int i = 0; i < pResSize; i++)
					txt += "[" + (i + 1).ToString() + "] 0x" + (resBuf[i]).ToString("x2") + "\n";
				MessageBox.Show("正常に受信しました。\n\n" + txt, "受信結果",
					MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
			else
			{
				MessageBox.Show("受信に失敗しました。", "受信結果",
					MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		// 40連LEDアレイモジュール開始設定
		private void start_setup()
		{
			// ピークバー無しにする
			System.Threading.Thread.Sleep(1);
			i2c_text_write(m_nSlave, "02 80 00 00 28 28 00");
			System.Threading.Thread.Sleep(1);
		}

		// 40連LEDアレイモジュール終了設定
		private void end_setup()
		{
			// ピークバー有りに戻す
			System.Threading.Thread.Sleep(1);
			i2c_text_write(m_nSlave, "03 08 64");
			System.Threading.Thread.Sleep(1);
		}
	}
}
