﻿// =============================================================================
// KitaLab USB to I2C モジュール サンプルプログラム２
// -----------------------------------------------------------------------------
// [動作OS]
// ・Windows XP (SP3)
// ・Windows 7 (32bit/64bit)
// [使用ツール/媒体]
// ・Microsoft Visual C# (開発はC#2008 Professional)
// ・KitaLab製 USB to I2Cモジュール
// ・KitaLab製 10桁7segモジュール×１個
// -----------------------------------------------------------------------------
// [使用方法]
// １．USB to I2C モジュールに10桁7segモジュールを接続する。
// ２．USB to I2C モジュールをPCに接続する。
//     (HIDデバイスとして認識されるのでドライバ不要)
// ３．test02.exeを起動する。
// ４．「開始」ボタンを押下して10桁7segモジュールに時計が表示されれば動作OK。
// -----------------------------------------------------------------------------
// Copyright(C) 2013 KitaLab
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace test02
{
	public partial class Form1 : Form
	{
		// USBtoI2C DLL関数準備
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_buff_write(byte slave, byte length, byte[] data);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_text_write(byte slave, string text);

		// 10桁7segモジュール点灯パターン定義
		byte[] digit = new byte[11]{
			0x3f,	// 0	 A   0b.GFEDCBA
			0x06,	// 1	F B
			0x5b,	// 2	 G
			0x4f,	// 3	E C
			0x66,	// 4	 D.
			0x6d,	// 5
			0x7d,	// 6
			0x27,	// 7
			0x7f,	// 8
			0x6f,	// 9
			0x80,	// .
		};

		// メンバ変数
		int m_nBackHour = -1;
		int m_nBackMinute = 0;
		int m_nBackSecond = 0;
		bool m_bEnable = false;
		byte m_nLuxPos = 0x20;
		byte m_nLux = 0x0f;
		byte m_nSlave = 0xC0;

		// コンストラクタ
		public Form1()
		{
			InitializeComponent();

			// dateTimePicker書式設定
			dateTimePicker1.Format = DateTimePickerFormat.Time;

			// スレーブアドレスコンボボックス準備
			for (byte i = 0x10; i < 0xf0; i += 2)
			{
				int idx = comboBox1.Items.Add( new CBItem( i, "0x" + (i).ToString("x2") ) );
				if (i == m_nSlave) comboBox1.SelectedIndex = idx;
			}

			// タイマー無効
			InitTimer(false);
		}

		// [event] タイマーイベント
		private void timer1_Tick(object sender, EventArgs e)
		{
			// 現在日時取得
			DateTime dt = DateTime.Now;

			// 開始前に一発呼ばれるので初期化に使用
			if (m_nBackHour < 0)
			{
				m_nBackHour = dt.Hour;
				m_nBackMinute = dt.Minute;
				m_nBackSecond = dt.Second;
			}

			// 輝度初期化
			if (m_nBackHour != dt.Hour || m_nBackMinute != dt.Minute || m_nBackSecond != dt.Second)
			{
				// 初期化位置決定 (10桁7segモジュールの輝度情報は0x20～0x29の10桁分)
				if (m_nBackHour != dt.Hour)
					m_nLuxPos = 0x21;
				else if (m_nBackMinute != dt.Minute)
					m_nLuxPos = 0x23;
				else if (m_nBackSecond != dt.Second)
					m_nLuxPos = 0x26;

				// 輝度ゼロ
				m_nLux = 0;
			}

			// フェードイン
			if (m_nLux <= 0x0f)
			{
				// I2C一括送信 / 7segモジュールの各桁の輝度更新
				byte[] send = new byte[11];
				send[0] = m_nLuxPos;
				for (int i = 1; i < 11; i++) send[i] = m_nLux;
				System.Threading.Thread.Sleep(10);
				i2c_buff_write(m_nSlave, (byte)(0x29 - m_nLuxPos + 1), send);

				// 輝度更新
				m_nLux++;
			}

			// 時間情報送信
			if (m_nBackHour != dt.Hour || m_nBackMinute != dt.Minute || m_nBackSecond != dt.Second)
			{
				// I2C送信データ作成
				byte[] send = new byte[11] {
					0x00,							// ポイントアドレス0x00
					0x00,							// １桁目(空き)
					digit[dt.Hour / 10 % 10],		// ２桁目(時)
					digit[dt.Hour % 10],			// ３桁目(時)
					digit[0x0a],					// ４桁目(．)
					digit[dt.Minute / 10 % 10],		// ５桁目(分)
					digit[dt.Minute % 10],			// ６桁目(分)
					digit[0x0a],					// ７桁目(．)
					digit[dt.Second / 10 % 10],		// ８桁目(秒)
					digit[dt.Second % 10],			// ９桁目(秒)
					0x00};							// 10桁目(空き)
				// 「時」が１桁の時は２桁目を消去
				if ((dt.Hour / 10 % 10) == 0)
					send[2] = 0;

				// I2C一括送信
				i2c_buff_write(m_nSlave, (byte)send.Length, send);

				// dateTimePicker更新
				dateTimePicker1.Value = dt;

				// 時間待避
				m_nBackHour = dt.Hour;
				m_nBackMinute = dt.Minute;
				m_nBackSecond = dt.Second;
			}
		}

		// [event] startボタン押下
		private void button1_Click(object sender, EventArgs e)
		{
			// タイマー開始
			InitTimer((m_bEnable)?false:true);
		}

		// タイマーON/OFF処理
		private void InitTimer(bool sw)
		{
			// ターゲットスレーブアドレス取得
			if (comboBox1.SelectedIndex < 0) return;
			CBItem item = (CBItem)comboBox1.SelectedItem;
			m_nSlave = item.Address;

			// 初期化
			if (sw)
			{
				m_nLux = 0x0f;
				m_nLuxPos = 0x20;
				m_nBackHour = -1;
				m_nBackMinute = -1;
				m_nBackSecond = -1;
			}

			// コントロール制御
			m_bEnable = sw;
			comboBox1.Enabled = !sw;
			button1.Text = (sw) ? "停止" : "送信開始";

			// タイマー有効/無効
			timer1.Interval = 40;
			timer1.Enabled = sw;
		}
	}

	// スレーブアドレスコンボボックスアイテムクラス
	public class CBItem
	{
		private byte m_address = 0;
		private string m_name = "";

		public CBItem(byte address, string name)
        {
			m_address = address;
            m_name = name;
        }
		public byte Address
		{
			set
			{
				m_address = value;
			}
			get
			{
				return m_address;
			}
		}
		public string Name
		{
			set
			{
				m_name = value;
			}
			get
			{
				return m_name;
			}
		}
		public override string ToString()
		{
			return m_name;
		}
	}
}
