﻿// =============================================================================
// KitaLab USB to I2C モジュール サンプルプログラム３
// 現在のメモリの空き容量をバイト単位でリアルタイムに表示します。
// 最大9GBまで表示されます。(10桁目がギガ)
// -----------------------------------------------------------------------------
// [動作OS]
// ・Windows XP (SP3)
// ・Windows 7 (32bit/64bit)
// [使用ツール/媒体]
// ・Microsoft Visual C# (開発はC#2008 Professional)
// ・KitaLab製 USB to I2Cモジュール
// ・KitaLab製 10桁7segモジュール×１個
// -----------------------------------------------------------------------------
// [使用方法]
// １．USB to I2C モジュールに10桁7segモジュールを接続する。
// ２．USB to I2C モジュールをPCに接続する。
//     (HIDデバイスとして認識されるのでドライバ不要)
// ３．test03.exeを起動する。
// ４．「送信開始」ボタンを押下して10桁7segモジュールに数値が表示されれば動作OK。
// -----------------------------------------------------------------------------
// Copyright(C) 2013 KitaLab
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace test03
{
	public partial class Form1 : Form
	{
		// USBtoI2C DLL関数準備
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_buff_write(byte slave, byte length, byte[] data);
		[DllImport("Usb2I2c.dll")]
		static extern bool i2c_text_write(byte slave, string text);

		// 10桁7segモジュール点灯パターン定義
		byte[] digit = new byte[11]{
			0x3f,	// 0	 A   0b.GFEDCBA
			0x06,	// 1	F B
			0x5b,	// 2	 G
			0x4f,	// 3	E C
			0x66,	// 4	 D.
			0x6d,	// 5
			0x7d,	// 6
			0x27,	// 7
			0x7f,	// 8
			0x6f,	// 9
			0x80,	// .
		};

		// メンバ変数
		System.Diagnostics.PerformanceCounter cPC = new System.Diagnostics.PerformanceCounter();
		bool m_bEnable = false;
		long m_nBackVal = 0;

		// コンストラクタ
		public Form1()
		{
			InitializeComponent();

			// メモリ空き容量取得設定
			cPC.CategoryName = "memory";
			cPC.CounterName = "Available Bytes";
			cPC.MachineName = ".";
			cPC.NextValue();

			// タイマー無効
			InitTimer(false);
		}

		// [event] タイマーイベント
		private void timer1_Tick(object sender, EventArgs e)
		{
			// 現在の物理メモリの空き容量をバイト単位で取得
			long nwk = (long)cPC.NextValue();
			if (m_nBackVal == nwk) return;
			m_nBackVal = nwk;

			// 表示情報作成
			byte[] send = new byte[11];
			send[10] = digit[0];
			for (int i = 0; nwk > 0; i++)
			{
				send[10 - i] = digit[nwk % 10];
				nwk = (int)(nwk / 10);
				if (i > 0 && i % 3 == 0)
					send[10 - i] |= 0x80;
			}

			// I2C送信
			i2c_buff_write(0xC0, 11, send);
		}

		// [event] startボタン押下
		private void button1_Click(object sender, EventArgs e)
		{
			// タイマー開始
			InitTimer((m_bEnable) ? false : true);
		}

		// タイマーON/OFF処理
		private void InitTimer(bool sw)
		{
			// コントロール制御
			m_bEnable = sw;
			button1.Text = (sw) ? "停止" : "送信開始";

			// タイマー有効/無効
			timer1.Interval = 10;
			timer1.Enabled = sw;
		}
	}
}
